import time
from multiprocessing import Process, Queue

class ProcessPool(object):
    
    def __init__(self, func, process_count=5):
        self.func = func
        self.process_count = process_count
        self.queue = Queue()
        self._create_processes()
    
    def _create_processes(self):
        for i in range(0, self.process_count):
            process = Process(target=self.func, args=[self.queue])
            process.daemon = True
            process.start()



import smtplib
import time

msg = """From: foo@localhost
To: bar@localhost

Here's my message!
"""
count = 0
while True:
    try:
        count += 1
        if (count % 10) == 0:
            server = smtplib.SMTP('localhost', port=465)
        else:
            server = smtplib.SMTP_SSL('localhost', port=465)
        
        server.set_debuglevel(1)
        server.login('bcoe', 'foobar')
        server.sendmail('foo@localhost', ['bar@localhost'], msg)
        server.quit()
    except Exception as e:
        print(e)
    time.sleep(0.05)



from secure_smtpd import SMTPServer

class SecureSMTPServer(SMTPServer):
    def process_message(self, peer, mailfrom, rcpttos, message_data):
        pass
        
server = SecureSMTPServer(('0.0.0.0', 1025), None)
server.run()



import smtplib

msg = """From: foo@localhost
To: bar@localhost

Here's my message!
"""

server = smtplib.SMTP_SSL('localhost', port=1025)
server.set_debuglevel(1)
server.login('bcoe', 'foobar')
server.sendmail('foo@localhost', ['bar@localhost'], msg)
server.quit()



import smtplib, time

messages_sent = 0.0
start_time = time.time()
msg = open('examples/benchmarking/benchmark.eml').read()

while True:
    
    if (messages_sent % 10) == 0:
        current_time = time.time()
        print('%s msg-written/sec' % (messages_sent / (current_time - start_time)))
    
    server = smtplib.SMTP('localhost', port=1025)
    server.sendmail('foo@localhost', ['bar@localhost'], msg)
    server.quit()
    
    messages_sent += 1.0



import logging
from secure_smtpd import SMTPServer, FakeCredentialValidator, LOG_NAME

class SSLSMTPServer(SMTPServer):
    def process_message(self, peer, mailfrom, rcpttos, message_data):
        print(message_data)

logger = logging.getLogger( LOG_NAME )
logger.setLevel(logging.INFO)

server = SSLSMTPServer(
    ('0.0.0.0', 1025),
    None,
    require_authentication=True,
    ssl=True,
    certfile='examples/server.crt',
    keyfile='examples/server.key',
    credential_validator=FakeCredentialValidator(),
    maximum_execution_time = 1.0
    )

server.run()



import secure_smtpd
import ssl, smtpd, asyncore, socket, logging, signal, time, sys

from .smtp_channel import SMTPChannel
from asyncore import ExitNow
from .process_pool import ProcessPool
from ssl import SSLError
try:
    from Queue import Empty
except ImportError:
    # We're on python3
    from queue import Empty

class SMTPServer(smtpd.SMTPServer):

    def __init__(self, localaddr, remoteaddr, ssl=False, certfile=None, keyfile=None, ssl_version=ssl.PROTOCOL_SSLv23, require_authentication=False, credential_validator=None, maximum_execution_time=30, process_count=5):
        smtpd.SMTPServer.__init__(self, localaddr, remoteaddr)
        self.logger = logging.getLogger( secure_smtpd.LOG_NAME )
        self.certfile = certfile
        self.keyfile = keyfile
        self.ssl_version = ssl_version
        self.subprocesses = []
        self.require_authentication = require_authentication
        self.credential_validator = credential_validator
        self.ssl = ssl
        self.maximum_execution_time = maximum_execution_time
        self.process_count = process_count
        self.process_pool = None

    def handle_accept(self):
        self.process_pool = ProcessPool(self._accept_subprocess, process_count=self.process_count)
        self.close()

    def _accept_subprocess(self, queue):
        while True:
            try:
                self.socket.setblocking(1)
                pair = self.accept()
                map = {}

                if pair is not None:

                    self.logger.info('_accept_subprocess(): smtp connection accepted within subprocess.')

                    newsocket, fromaddr = pair
                    newsocket.settimeout(self.maximum_execution_time)

                    if self.ssl:
                        newsocket = ssl.wrap_socket(
                            newsocket,
                            server_side=True,
                            certfile=self.certfile,
                            keyfile=self.keyfile,
                            ssl_version=self.ssl_version,
                        )
                    channel = SMTPChannel(
                        self,
                        newsocket,
                        fromaddr,
                        require_authentication=self.require_authentication,
                        credential_validator=self.credential_validator,
                        map=map
                    )

                    self.logger.info('_accept_subprocess(): starting asyncore within subprocess.')

                    asyncore.loop(map=map)

                    self.logger.error('_accept_subprocess(): asyncore loop exited.')
            except (ExitNow, SSLError):
                self._shutdown_socket(newsocket)
                self.logger.info('_accept_subprocess(): smtp channel terminated asyncore.')
            except Exception as e:
                self._shutdown_socket(newsocket)
                self.logger.error('_accept_subprocess(): uncaught exception: %s' % str(e))

    def _shutdown_socket(self, s):
        try:
            s.shutdown(socket.SHUT_RDWR)
            s.close()
        except Exception as e:
            self.logger.error('_shutdown_socket(): failed to cleanly shutdown socket: %s' % str(e))


    def run(self):
        asyncore.loop()
        if hasattr(signal, 'SIGTERM'):
            def sig_handler(signal,frame):
                self.logger.info("Got signal %s, shutting down." % signal)
                sys.exit(0)
            signal.signal(signal.SIGTERM, sig_handler)
        while 1:
            time.sleep(1)



import socket
import smtplib
import secure_smtpd
from .smtp_server import SMTPServer
from .store_credentials import StoreCredentials

class ProxyServer(SMTPServer):
    """Implements an open relay.  Inherits from secure_smtpd, so can handle
    SSL incoming.  Modifies attributes slightly:

    * if "ssl" is true accepts SSL connections inbound and connects via SSL
        outbound
    * adds "ssl_out_only", which can be set to True when "ssl" is False so that
        inbound connections are in plain text but outbound are in SSL
    * adds "debug", which if True copies all inbound messages to logger.info()
    * ignores any credential validators, passing any credentials upstream
    """
    def __init__(self, *args, **kwargs):
        self.ssl_out_only = False
        if 'ssl_out_only' in kwargs:
            self.ssl_out_only = kwargs.pop('ssl_out_only')

        self.debug = False
        if 'debug' in kwargs:
            self.debug = kwargs.pop('debug')

        kwargs['credential_validator'] = StoreCredentials()
        SMTPServer.__init__(self, *args, **kwargs)

    def process_message(self, peer, mailfrom, rcpttos, data):
        if self.debug:
            # ------------------------
            # stolen directly from stmpd.DebuggingServer
            inheaders = 1
            lines = data.split('\n')
            self.logger.info('---------- MESSAGE FOLLOWS ----------')
            for line in lines:
                # headers first
                if inheaders and not line:
                    self.logger.info('X-Peer: %s', peer[0])
                    inheaders = 0
                self.logger.info(line)
            self.logger.info('------------ END MESSAGE ------------')

        # ------------------------
        # following code is direct from smtpd.PureProxy
        lines = data.split('\n')
        # Look for the last header
        i = 0
        for line in lines:
            if not line:
                break
            i += 1
        lines.insert(i, 'X-Peer: %s' % peer[0])
        data = '\n'.join(lines)
        self._deliver(mailfrom, rcpttos, data)

    def _deliver(self, mailfrom, rcpttos, data):
        # ------------------------
        # following code is adapted from smtpd.PureProxy with modifications to
        # handle upstream SSL
        refused = {}
        try:
            if self.ssl or self.ssl_out_only:
                s = smtplib.SMTP_SSL()
            else:
                s = smtplib.SMTP()

            s.connect(self._remoteaddr[0], self._remoteaddr[1])
            if self.credential_validator.stored:
                # we had credentials passed in, use them
                s.login(
                    self.credential_validator.username,
                    self.credential_validator.password
                )
            try:
                refused = s.sendmail(mailfrom, rcpttos, data)
                if refused != {}:
                    self.logger.error('some connections refused %s', refused)
            finally:
                s.quit()
        except smtplib.SMTPRecipientsRefused as e:
            self.logger.exception('')
            refused = e.recipients
        except (socket.error, smtplib.SMTPException) as e:
            self.logger.exception('')

            # All recipients were refused.  If the exception had an associated
            # error code, use it.  Otherwise,fake it with a non-triggering
            # exception code.
            errcode = getattr(e, 'smtp_code', -1)
            errmsg = getattr(e, 'smtp_error', 'ignore')
            for r in rcpttos:
                refused[r] = (errcode, errmsg)
        return refused



import secure_smtpd.config
from secure_smtpd.config import LOG_NAME
from .smtp_server import SMTPServer
from .fake_credential_validator import FakeCredentialValidator
from .proxy_server import ProxyServer



import logging, sys
from logging.handlers import RotatingFileHandler
from logging import StreamHandler

LOG_NAME = 'secure-smtpd'

class Log(object):
    
    def __init__(self, log_name):
        self.log_name = log_name
        self.logger = logging.getLogger( self.log_name )
        self._remove_handlers()
        self._add_handler()
        self.logger.setLevel(logging.DEBUG)
    
    def _remove_handlers(self):
        for handler in self.logger.handlers:
            self.logger.removeHandler(handler)
    
    def _add_handler(self):
        try:
            handler = RotatingFileHandler(
                '/var/log/%s.log' % self.log_name,
                maxBytes=10485760,
                backupCount=3
            )
            formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
            handler.setFormatter(formatter)
            self.logger.addHandler(handler)
        except IOError:
            self.logger.addHandler(StreamHandler(sys.stderr))

Log(LOG_NAME)


#!/usr/bin/env python
#from distutils.core import setup
from setuptools import setup, find_packages

with open('README.markdown') as f:
    long_description = f.read()

setup(
    name="secure-smtpd",
    version="3.0.0",
    description="Adds support for SSL, AUTH, and other goodies, to Petri Lehtinen's SMTPD library.",
    long_description=long_description,
    author="Benjamin Coe",
    author_email="bencoe@gmail.com",
    url="https://github.com/bcoe/secure-smtpd",
    keywords='secure ssl auth smtp smtpd server',
    license='ISC',
    packages = find_packages(),
    install_requires = [
        'argparse'
    ],
    tests_require=[
        'nose'
    ],
    include_package_data=True,
    zip_safe=False,
    classifiers=[ # See: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Topic :: Communications :: Email :: Mail Transport Agents',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'License :: OSI Approved :: ISC License (ISCL)',
    ],
)



class StoreCredentials(object):
    def __init__(self):
        self.stored = False
        self.username = None
        self.password = None

    def validate(self, username, password):
        self.stored = True
        self.username = username
        self.password = password
        return True



#!/usr/bin/env python
import argparse
import sys
from secure_smtpd import ProxyServer

def run(cmdargs):
    args = [
        (cmdargs.localhost, cmdargs.localport),
        (cmdargs.remotehost, cmdargs.remoteport)
    ]
    kwargs = {}

    if cmdargs.sslboth:
        kwargs['ssl'] = True
        if not cmdargs.certfile or not cmdargs.keyfile:
            print ('You need to specify a valid certificate file and a key file!')
            sys.exit(1)
        kwargs['certfile'] = cmdargs.certfile
        kwargs['keyfile'] = cmdargs.keyfile
    elif cmdargs.sslout:
        kwargs['ssl_out_only'] = True

    if not cmdargs.quiet:
        kwargs['debug'] = True

    server = ProxyServer(*args, **kwargs)
    server.run()

parser = argparse.ArgumentParser(description='mail relay tool')

parser.add_argument(
    '--localhost',
    default='127.0.0.1',
    help='Local address to attach to for receiving mail.  Defaults to 127.0.0.1'
)
    
parser.add_argument(
    '--localport',
    default=1025,
    type=int,
    help='Local port to attach to for receiving mail.  Defaults to 1025'
)

parser.add_argument(
    '--remotehost',
    required=True,
    help='Address of the remote server for connection.'
)

parser.add_argument(
    '--remoteport',
    default=25,
    type=int, 
    help='Port of the remote server for connection.  Defaults to 25'
)

parser.add_argument(
    '--quiet',
    action='store_true',
    help='Use this to turn off the message printing'
)

group = parser.add_mutually_exclusive_group()

group.add_argument(
    '--sslboth',
    action='store_true', 
    help='Use this parameter if both the inbound and outbound connections should use SSL'
)
    
group.add_argument(
    '--sslout',
    action='store_true', 
    help='Use this parameter if inbound connection is plain but the outbound connection uses SSL'
)

parser.add_argument(
    '--certfile',
    help='Certificate file to use for inbound SSL connections'
)

parser.add_argument(
    '--keyfile',
    help='Key file to use for inbound SSL connections'
)

args = parser.parse_args()

print('Starting ProxyServer')
print('local: %s:%s' % (args.localhost, args.localport))
print('remote: %s:%s' % (args.remotehost, args.remoteport))
print('sslboth: ', args.sslboth)
print('sslout: ', args.sslout)
print
run(args)
